/* Taken from https://github.com/djpohly/dwl/issues/466 */
#define COLOR(hex)    { ((hex >> 24) & 0xFF) / 255.0f, \
                        ((hex >> 16) & 0xFF) / 255.0f, \
                        ((hex >> 8) & 0xFF) / 255.0f, \
                        (hex & 0xFF) / 255.0f }
/* appearance */
static const int sloppyfocus                = 0;  /* focus follows mouse */
static const int bypass_surface_visibility  = 0;  /* 1 means idle inhibitors will disable idle tracking even if it's surface isn't visible  */
static const unsigned int borderpx          = 10;  /* border pixel of windows */
static const int lockfullscreen             = 1;  /* 1 will force focus on the fullscreen window */
static const float rootcolor[]              = COLOR(0x4d4d4d66);
static const float bordercolor[]            = COLOR(0x00000000);
static const float focuscolor[]             = COLOR(0x12121212);
static const float urgentcolor[]            = COLOR(0xff0000ff);
/* To conform the xdg-protocol, set the alpha to zero to restore the old behavior */
static const float fullscreen_bg[]          = COLOR(0x121212ff);

/* tagging - TAGCOUNT must be no greater than 31 */
#define TAGCOUNT (9)

/* control variables */
static int cfg_disable_touchpad = 1;
static const char *cfg_touchpad_device_str = "Synaptic";

/* logging */
static int log_level = WLR_ERROR;

static const Rule rules[] = {
	/* app_id     title       tags mask     isfloating   monitor */
	/* examples:
	{ "firefox",  NULL,       1 << 8,       0,           -1 },
	*/
	{ "Gimp",     NULL,       0,            1,           -1 },
};

/* layout(s) */
static const Layout layouts[] = {
	/* symbol     arrange function */
	{ "[]=",      tile },
	{ "><>",      NULL },    /* no layout function means floating behavior */
	{ "[M]",      monocle },
};

/* monitors */
/* NOTE: ALWAYS add a fallback rule, even if you are completely sure it won't be used */
static const MonitorRule monrules[] = {
	/* name       mfact  nmaster scale layout       rotate/reflect                x    y */
	/* example of a HiDPI laptop monitor:
	{ "eDP-1",    0.5f,  1,      2,    &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL,   -1,  -1 },
	*/
	/* defaults */
	{ NULL,       0.55f, 1,      1,    &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL,   -1,  -1 },
};

/* keyboard */
static const struct xkb_rule_names xkb_rules = {
	/* can specify fields: rules, model, layout, variant, options
     * Base rules in /usr/share/X11/xkb/rules/base.lst */
	/* example:
	.options = "ctrl:nocaps",
	*/
	.options = "caps:none,compose:rctrl",
};

static const int repeat_rate = 25;
static const int repeat_delay = 600;

/* Trackpad */
static const int tap_to_click = 1;
static const int tap_and_drag = 1;
static const int drag_lock = 1;
static const int natural_scrolling = 1;
static const int disable_while_typing = 1;
static const int left_handed = 0;
static const int middle_button_emulation = 0;
/* You can choose between:
LIBINPUT_CONFIG_SCROLL_NO_SCROLL
LIBINPUT_CONFIG_SCROLL_2FG
LIBINPUT_CONFIG_SCROLL_EDGE
LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN
*/
static const enum libinput_config_scroll_method scroll_method = LIBINPUT_CONFIG_SCROLL_2FG;

/* You can choose between:
LIBINPUT_CONFIG_CLICK_METHOD_NONE
LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS
LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER
*/
static const enum libinput_config_click_method click_method = LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS;

/* You can choose between:
LIBINPUT_CONFIG_SEND_EVENTS_ENABLED
LIBINPUT_CONFIG_SEND_EVENTS_DISABLED
LIBINPUT_CONFIG_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE
*/
static const uint32_t send_events_mode = LIBINPUT_CONFIG_SEND_EVENTS_ENABLED;

/* You can choose between:
LIBINPUT_CONFIG_ACCEL_PROFILE_FLAT
LIBINPUT_CONFIG_ACCEL_PROFILE_ADAPTIVE
*/
static const enum libinput_config_accel_profile accel_profile = LIBINPUT_CONFIG_ACCEL_PROFILE_ADAPTIVE;
static const double accel_speed = 0.0;
/* You can choose between:
LIBINPUT_CONFIG_TAP_MAP_LRM -- 1/2/3 finger tap maps to left/right/middle
LIBINPUT_CONFIG_TAP_MAP_LMR -- 1/2/3 finger tap maps to left/middle/right
*/
static const enum libinput_config_tap_button_map button_map = LIBINPUT_CONFIG_TAP_MAP_LRM;

/* If you want to use the windows key for MODKEY, use WLR_MODIFIER_LOGO */
/* #define MODKEY WLR_MODIFIER_LOGO */
#define MODKEY WLR_MODIFIER_ALT
#define TAGKEYS(KEY,SKEY,TAG) \
	{ MODKEY,                    KEY,            view,            {.ui = 1 << TAG} }, \
	{ MODKEY|WLR_MODIFIER_CTRL,  KEY,            toggleview,      {.ui = 1 << TAG} }, \
	{ MODKEY|WLR_MODIFIER_SHIFT, SKEY,           tag,             {.ui = 1 << TAG} }, \
	{ MODKEY|WLR_MODIFIER_CTRL|WLR_MODIFIER_SHIFT,SKEY,toggletag, {.ui = 1 << TAG} }

#define AD_STR(a) #a
#define AD_CONCAT(a, b) AD_STR(a ## b)
#define ACTKEYS_DECL(FUNCP) \
	static const char *key_cmd_ ## FUNCP[]      = { "dwl_act", #FUNCP,     NULL }; \
	static const char *key_cmd_S_ ## FUNCP[]    = { "dwl_act", AD_CONCAT(S_,FUNCP),   NULL }; \
	static const char *key_cmd_C_ ## FUNCP[]    = { "dwl_act", AD_CONCAT(C_,FUNCP),   NULL }; \
	static const char *key_cmd_SC_ ## FUNCP[]   = { "dwl_act", AD_CONCAT(SC_,FUNCP),  NULL }

#define ACTKEYS_MAP(KEY,FUNCP) \
	{ 0,                                     KEY,    spawn,  {.v =  key_cmd_ ## FUNCP} }, \
	{ WLR_MODIFIER_CTRL,                     KEY,    spawn,  {.v =  key_cmd_C_ ## FUNCP} }, \
	{ WLR_MODIFIER_SHIFT,                    KEY,    spawn,  {.v =  key_cmd_S_ ## FUNCP} }, \
	{ WLR_MODIFIER_CTRL|WLR_MODIFIER_SHIFT,  KEY,    spawn,  {.v =  key_cmd_SC_ ## FUNCP} }

/* helper for spawning shell commands in the pre dwm-5.0 fashion */
#define SHCMD(cmd) { .v = (const char*[]){ "/bin/sh", "-c", cmd, NULL } }

/* commands */
static const char *webcmd[]                 = { "dwl_web",                  NULL };
static const char *mailcmd[]                = { "dwl_email",                NULL };
static const char *termcmd[]                = { "dwl_term",                 NULL };
static const char *alttermcmd[]             = { "dwl_alt_term",             NULL };
static const char *launchcmd[]              = { "dwl_dmenu",                NULL };
static const char *dispcmd[]                = { "wdisplays",                NULL };
static const char *lockcmd[]                = { "swaylock",                 NULL };

/* Function key commands */
ACTKEYS_DECL(vol_mute);
ACTKEYS_DECL(vol_lower);
ACTKEYS_DECL(vol_raise);
ACTKEYS_DECL(mic_mute);
ACTKEYS_DECL(bright_low);
ACTKEYS_DECL(bright_high);
ACTKEYS_DECL(disp);
ACTKEYS_DECL(wlan);
ACTKEYS_DECL(webcam);
ACTKEYS_DECL(tools);
ACTKEYS_DECL(search);
ACTKEYS_DECL(launch);
ACTKEYS_DECL(explore);
ACTKEYS_DECL(print);
ACTKEYS_DECL(media_toggle);
ACTKEYS_DECL(media_stop);
ACTKEYS_DECL(media_prev);
ACTKEYS_DECL(media_next);

static const Key keys[] = {
	/* Note that Shift changes certain key codes: c -> C, 2 -> at, etc. */
	/* modifier                  key                                function            argument */
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_Return,                    spawn,              {.v = termcmd} },
	{ MODKEY|WLR_MODIFIER_CTRL|WLR_MODIFIER_SHIFT,
                                 XKB_KEY_Return,                    spawn,              {.v = alttermcmd} },
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_P,                         spawn,              {.v = launchcmd} },
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_N,                         spawn,              {.v = dispcmd} },
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_W,                         spawn,              {.v = webcmd} },
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_E,                         spawn,              {.v = mailcmd} },
	{ MODKEY,                    XKB_KEY_b,                         togglebar,          {0}},
	{ MODKEY,                    XKB_KEY_j,                         focusstack,         {.i = +1} },
	{ MODKEY,                    XKB_KEY_k,                         focusstack,         {.i = -1} },
	{ MODKEY,                    XKB_KEY_i,                         incnmaster,         {.i = +1} },
	{ MODKEY,                    XKB_KEY_d,                         incnmaster,         {.i = -1} },
	{ MODKEY,                    XKB_KEY_h,                         setmfact,           {.f = -0.05f} },
	{ MODKEY,                    XKB_KEY_l,                         setmfact,           {.f = +0.05f} },
	{ MODKEY,                    XKB_KEY_Return,                    zoom,               {0} },
	{ MODKEY,                    XKB_KEY_Tab,                       view,               {0} },
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_C,                         killclient,         {0} },
	{ MODKEY,                    XKB_KEY_t,                         setlayout,          {.v = &layouts[0]} },
	{ MODKEY,                    XKB_KEY_f,                         setlayout,          {.v = &layouts[1]} },
	{ MODKEY,                    XKB_KEY_m,                         setlayout,          {.v = &layouts[2]} },
	{ MODKEY,                    XKB_KEY_space,                     setlayout,          {0} },
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_space,                     togglefloating,     {0} },
	{ MODKEY,                    XKB_KEY_e,                         togglefullscreen,
                                                                                        {0} },
	{ MODKEY,                    XKB_KEY_0,                         view,               {.ui = ~0} },
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_parenright,                tag,                {.ui = ~0} },
	{ MODKEY,                    XKB_KEY_comma,                     focusmon,           {.i = WLR_DIRECTION_LEFT} },
	{ MODKEY,                    XKB_KEY_period,                    focusmon,           {.i = WLR_DIRECTION_RIGHT} },
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_less,                      tagmon,             {.i = WLR_DIRECTION_LEFT} },
	{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_greater,                   tagmon,             {.i = WLR_DIRECTION_RIGHT} },
	{ MODKEY|WLR_MODIFIER_CTRL|WLR_MODIFIER_SHIFT, XKB_KEY_Q,       quit,               {0} },
	{ MODKEY|WLR_MODIFIER_CTRL|WLR_MODIFIER_SHIFT, XKB_KEY_L,       spawn,              {.v = lockcmd} },
	{ WLR_MODIFIER_LOGO, XKB_KEY_l,                                 spawn,              {.v = lockcmd } },
	{ WLR_MODIFIER_LOGO, XKB_KEY_p,                                 spawn,              {.v = key_cmd_disp } },

	/* Ctrl-Alt-Backspace and Ctrl-Alt-Fx used to be handled by X server */
	{ WLR_MODIFIER_CTRL|WLR_MODIFIER_ALT,XKB_KEY_Terminate_Server,  quit,               {0} },

    /* Media control keys without device specific action buttons */
	{ WLR_MODIFIER_LOGO, XKB_KEY_space,                             spawn,              {.v = key_cmd_media_toggle } },
	{ WLR_MODIFIER_LOGO, XKB_KEY_s,                                 spawn,              {.v = key_cmd_media_stop } },
	{ WLR_MODIFIER_LOGO, XKB_KEY_period,                            spawn,              {.v = key_cmd_media_next } },
	{ WLR_MODIFIER_LOGO, XKB_KEY_comma,                             spawn,              {.v = key_cmd_media_prev } },
	{ WLR_MODIFIER_LOGO, XKB_KEY_a,                                 spawn,              {.v = key_cmd_vol_mute } },
	{ WLR_MODIFIER_LOGO, XKB_KEY_z,                                 spawn,              {.v = key_cmd_vol_lower } },
	{ WLR_MODIFIER_LOGO, XKB_KEY_q,                                 spawn,              {.v = key_cmd_vol_raise } },

	/* Action buttons */
	ACTKEYS_MAP(XKB_KEY_XF86AudioMute,                              vol_mute),
	ACTKEYS_MAP(XKB_KEY_XF86AudioLowerVolume,                       vol_lower),
	ACTKEYS_MAP(XKB_KEY_XF86AudioRaiseVolume,                       vol_raise),
	ACTKEYS_MAP(XKB_KEY_XF86AudioMicMute,                           mic_mute),
	ACTKEYS_MAP(XKB_KEY_XF86MonBrightnessDown,                      bright_low),
	ACTKEYS_MAP(XKB_KEY_XF86MonBrightnessUp,                        bright_high),
	ACTKEYS_MAP(XKB_KEY_XF86Display,                                disp),
	ACTKEYS_MAP(XKB_KEY_XF86WLAN,                                   wlan),
	ACTKEYS_MAP(XKB_KEY_XF86WebCam,                                 webcam),
	ACTKEYS_MAP(XKB_KEY_XF86Tools,                                  tools),
	ACTKEYS_MAP(XKB_KEY_XF86Search,                                 search),
	ACTKEYS_MAP(XKB_KEY_XF86LaunchA,                                launch),
	ACTKEYS_MAP(XKB_KEY_XF86Explorer,                               explore),
	ACTKEYS_MAP(XKB_KEY_Print,                                      print),
	ACTKEYS_MAP(XKB_KEY_XF86AudioPlay,                              media_toggle),
	ACTKEYS_MAP(XKB_KEY_XF86AudioStop,                              media_stop),
	ACTKEYS_MAP(XKB_KEY_XF86AudioPrev,                              media_prev),
	ACTKEYS_MAP(XKB_KEY_XF86AudioNext,                              media_next),
	/* { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_space,      toggle_touch_input_device,
                                                                    {0} },*/
	TAGKEYS(          XKB_KEY_1, XKB_KEY_exclam,                    0),
	TAGKEYS(          XKB_KEY_2, XKB_KEY_at,                        1),
	TAGKEYS(          XKB_KEY_3, XKB_KEY_numbersign,                2),
	TAGKEYS(          XKB_KEY_4, XKB_KEY_dollar,                    3),
	TAGKEYS(          XKB_KEY_5, XKB_KEY_percent,                   4),
	TAGKEYS(          XKB_KEY_6, XKB_KEY_asciicircum,               5),
	TAGKEYS(          XKB_KEY_7, XKB_KEY_ampersand,                 6),
	TAGKEYS(          XKB_KEY_8, XKB_KEY_asterisk,                  7),
	TAGKEYS(          XKB_KEY_9, XKB_KEY_parenleft,                 8),

	/* Ctrl-Alt-Fx is used to switch to another VT, if you don't know what a VT is
	 * do not remove them.
	 */
#define CHVT(n) { WLR_MODIFIER_CTRL|WLR_MODIFIER_ALT,XKB_KEY_XF86Switch_VT_##n, chvt, {.ui = (n)} }
	CHVT(1), CHVT(2), CHVT(3), CHVT(4), CHVT(5), CHVT(6),
	CHVT(7), CHVT(8), CHVT(9), CHVT(10), CHVT(11), CHVT(12),
};

static const Button buttons[] = {
	{ MODKEY, BTN_LEFT,   moveresize,     {.ui = CurMove} },
	{ MODKEY, BTN_MIDDLE, togglefloating, {0} },
	{ MODKEY, BTN_RIGHT,  moveresize,     {.ui = CurResize} },
};
